//------------------------------------------------------------
// Copyright (c) Microsoft Corporation.  All rights reserved.
//------------------------------------------------------------

// This file isn't generated, but this comment is necessary to exclude it from StyleCop analysis.
// <auto-generated/>

using System;

namespace Microsoft.Data.Encryption.Cryptography.Serializers
{
    /// <summary>
    /// Contains the methods for serializing and deserializing <see cref="DateTimeOffset"/>? type data objects
    /// that is compatible with the Always Encrypted feature in SQL Server and Azure SQL Database.
    /// </summary>
    internal class SqlNullableDateTimeOffsetSerializer : Serializer<DateTimeOffset?>
    {
        private const int DefaultScale = 7;
        private readonly SqlDateTimeOffsetSerializer serializer;

        /// <summary>
        /// The <see cref="Identifier"/> uniquely identifies a particular Serializer implementation.
        /// </summary>
        public override string Identifier => "SQL_DateTimeOffset_Nullable";

        /// <summary>
        /// Gets or sets the number of decimal places to which Value is resolved. The default is 7.
        /// </summary>
        /// <exception cref="MicrosoftDataEncryptionException">
        /// Thrown when set to a value that is out of the valid range [0 - 7] for this setting.
        /// </exception>
        public int Scale
        {
            get => serializer.Scale;
            set => serializer.Scale = value;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SqlNullableDateTimeOffsetSerializer"/> class.
        /// </summary>
        /// <param name="scale">The number of decimal places to which Value is resolved.</param>
        /// <exception cref="MicrosoftDataEncryptionException">
        /// Thrown when set to a value that is out of the valid range [0 - 7] for this setting.
        /// </exception>
        public SqlNullableDateTimeOffsetSerializer(int scale = DefaultScale)
        {
            serializer = new SqlDateTimeOffsetSerializer(scale);
        }


        /// <summary>
        /// Deserializes the provided <paramref name="bytes"/>
        /// </summary>
        /// <param name="bytes">The data to be deserialized</param>
        /// <returns>The serialized data</returns>
        /// <exception cref="MicrosoftDataEncryptionException">
        /// The length of <paramref name="bytes"/> is less than 10.
        /// </exception>
        public override DateTimeOffset? Deserialize(byte[] bytes)
        {
            return bytes.IsNull() ? (DateTimeOffset?)null : serializer.Deserialize(bytes);
        }

        /// <summary>
        /// Serializes the provided <paramref name="value"/>
        /// </summary>
        /// <param name="value">The value to be serialized</param>
        /// <returns>
        /// An array of bytes with length 10.
        /// </returns>
        public override byte[] Serialize(DateTimeOffset? value)
        {
            return value.IsNull() ? null : serializer.Serialize(value.Value);
        }
    }
}
